/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "md5sumfile.h"

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qregexp.h>

namespace KDigest {

/**
@brief Default constructor
@param path Path of the SFV file
@param parent parent object (passed to DigestFile and then to QObject)
@param name object's name (passed to DigestFile and then to QObject)
*/
MD5SUMfile::MD5SUMfile(const QString &path, QObject *parent, const char *name)
	: DigestFile(parent, name)
{
	QFile f(path);
	if ( ! f.open(IO_ReadOnly) ) {
		///@todo Thrown an error
		return;
	}
	
	QTextStream ts(&f);
	
	QRegExp re("([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})[\\* ]{1} (.*)");
	QString line;
	while(!ts.atEnd())
	{
		line = ts.readLine();
		if ( re.search(line) == -1 )
			continue;
		
		uint32_t md5Bytes[4]; bool ok = false;
		md5Bytes[0] = re.cap(1).toUInt(&ok, 16); if ( ! ok ) continue;
		md5Bytes[1] = re.cap(2).toUInt(&ok, 16); if ( ! ok ) continue;
		md5Bytes[2] = re.cap(3).toUInt(&ok, 16); if ( ! ok ) continue;
		md5Bytes[3] = re.cap(4).toUInt(&ok, 16); if ( ! ok ) continue;
		
		MD5SUMentry *e = new MD5SUMentry;
		
		e->filename = re.cap(5);
		e->status = dsNotChecked;

		memcpy(e->md5, md5Bytes, sizeof(e->md5));
		memset(e->aMd5, 0, sizeof(e->aMd5));
		
		m_entries.append(e);
	}
	
	f.close();
}

MD5SUMfile::~MD5SUMfile()
{
}

/**
@brief Adds a file in the digest
@param filename Complete path of the file to add
*/
void MD5SUMfile::addFile(const QString &filename)
{
	m_entries.append( new MD5SUMentry( filename ) );
}

/**
@brief Gets the digest of the given opened file
@param f The file to get the digest of (opened)
*/
inline KMD5 &MD5SUMfile::MD5SUMentry::getDigest(QFile &f, KMD5 &context)
{
	context.reset();
	context.update(f);
	qApp->processEvents();
	
	return context;
}

/**
@brief Create a new SFVentry creating the digest for the given file
@param filepath Complete path of the file to digest
*/
MD5SUMfile::MD5SUMentry::MD5SUMentry(const QString &filepath)
{
	QDir filedir(filepath); filedir.cd("..");
	
	QFile f(filepath);
	if ( ! f.open(IO_ReadOnly) )
	{
		status = dsErrorAccess;
		return;
	}
	
	filename = filepath; filename.remove(filedir.path() + "/");
	
	KMD5 context(NULL, 0);
	
	QRegExp re("([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})");
	QString digest = getDigest(f, context).hexDigest();
	re.search(digest);
	
	uint32_t *tMd5 = reinterpret_cast<uint32_t*>(md5);
	
	tMd5[0] = re.cap(1).toUInt(NULL, 16);
	tMd5[1] = re.cap(2).toUInt(NULL, 16);
	tMd5[2] = re.cap(3).toUInt(NULL, 16);
	tMd5[3] = re.cap(4).toUInt(NULL, 16);
	f.close();
}

/**
@brief Do the dirt work on calculating the digest of a file
@param filename Complete filename of the entry
*/
bool MD5SUMfile::MD5SUMentry::calculateActual(const QString &filename)
{
	QFile chkFile(filename);
	if ( ! chkFile.open(IO_ReadOnly) )
	{
		status = dsErrorAccess;
		return false;
	}
	
	KMD5 context;
	// Doing this because it's much more simpler than a hton with a 128-bit value ;)
	QRegExp re("([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})");
	QString digest = getDigest(chkFile, context).hexDigest();
	re.search(digest);
	
	uint32_t *tMd5 = reinterpret_cast<uint32_t*>(aMd5);
	
	tMd5[0] = re.cap(1).toUInt(NULL, 16);
	tMd5[1] = re.cap(2).toUInt(NULL, 16);
	tMd5[2] = re.cap(3).toUInt(NULL, 16);
	tMd5[3] = re.cap(4).toUInt(NULL, 16);
	
	qApp->processEvents();
	
	if ( memcmp(aMd5, md5, sizeof(md5)) == 0 )
	{
		status = dsOk;
		return true;
	} else {
		status = dsBadDigest;
		return false;
	}
	chkFile.close();
}

/**
@brief Write the entry on the given file
@param file Opened file where to save the entry to
*/
void MD5SUMfile::MD5SUMentry::writeOnFile(QFile &file)
{
	QString entryLine = digest() + "  " + filename + "\n";
	file.writeBlock(entryLine, entryLine.length());
}

};
#include "md5sumfile.moc"
